#include "SAB.h"

SAB::SAB() : _screenConfig(128, 64, 0x3C),
_sdCardConfig(SPI_FULL_SPEED), 
_display(_screenConfig.getWidth(), 
_screenConfig.getHeight(), &Wire), 
_screenManager(_display), _rtc(), 
_rtcManager(_rtc), _sdCardManager(), 
_connectivityManager(NULL), 
//_webServerManager(80, &_sdCardManager),
_webServer(80, &_sdCardManager),
_ftpServer(21, &_sdCardManager, "ESP8266", "12345678"),
_pcf(0x27, Wire),
_ioManager(_pcf),
_taskSchedulerManager(_rtcManager),
_error(0)
{
  //We set the gpio up
  initGPIO();
  //Serial.begin(9600);
  Serial.begin(115200, SERIAL_8N1, SERIAL_TX_ONLY);
  delay(200);
  
  //We initialize the pins for the  I2C communication
  Wire.begin(_pinConfig.getI2C_sda(), _pinConfig.getI2C_scl());
  if(!_rtc.begin()) _error |= RTC_BEGIN_ERR;
  if(!_display.begin(SSD1306_SWITCHCAPVCC, _screenConfig.getAddress())){ _error |= DISP_BEGIN_ERR; }
  if(!_sdCardManager.begin(_pinConfig.getSPI_sdCard_cs(), _sdCardConfig.getSPISpeed())){ _error |= SDCARD_INIT_ERR; Serial.print("Failed to init SDCard : SPI_SPEED : "); Serial.print(_sdCardConfig.getSPISpeed()); Serial.print(" CS PIN : "); Serial.println(_pinConfig.getSPI_sdCard_cs());}
  _connectivityManager = new ConnectivityManager(_sdCardManager);
  if(!_pcf.begin()){_error |= IO_INIT_ERR;}
}

SAB::SAB(const PinMapping pinConfig, const ScreenConfig screenConfig, const SDCardConfig sdCardConfig, const unsigned int webServerPort) : _pinConfig(pinConfig), 
_screenConfig(screenConfig), 
_sdCardConfig(sdCardConfig), 
_display(_screenConfig.getWidth(), _screenConfig.getHeight(), &Wire), 
_screenManager(_display), 
_rtc(), 
_rtcManager(_rtc), 
_sdCardManager(), 
_connectivityManager(NULL), 
//_webServerManager(webServerPort, &_sdCardManager),
_webServer(webServerPort, &_sdCardManager),
_ftpServer(21, &_sdCardManager, "ESP8266", "12345678"),
_pcf(0x27, Wire), 
_ioManager(_pcf),
_taskSchedulerManager(_rtcManager),
_error(0)
{
  //We set the gpio up
  initGPIO();
  //Serial.begin(9600);
  Serial.begin(115200, SERIAL_8N1, SERIAL_TX_ONLY);
  delay(200);
  //We initialize the pins for the  I2C communication
  Wire.begin(_pinConfig.getI2C_sda(), _pinConfig.getI2C_scl());
  if(!_rtc.begin()) _error |= RTC_BEGIN_ERR;
  if(!_display.begin(SSD1306_SWITCHCAPVCC, _screenConfig.getAddress())) _error |= DISP_BEGIN_ERR;
  if(!_sdCardManager.begin(_pinConfig.getSPI_sdCard_cs(), _sdCardConfig.getSPISpeed())){ _error |= SDCARD_INIT_ERR; Serial.print("Failed to init SDCard : SPI_SPEED : "); Serial.print(_sdCardConfig.getSPISpeed()); Serial.print(" CS PIN : "); Serial.println(_pinConfig.getSPI_sdCard_cs());}
  _connectivityManager = new ConnectivityManager(_sdCardManager);
  if(!_pcf.begin()){_error |= IO_INIT_ERR;}
}

void SAB::initGPIO()
{
  pinMode(GPIO_0, INPUT);
}

ScreenManager& SAB::getScreenManager()
{
  return _screenManager;
}

RtcManager& SAB::getRtcManager()
{
  return _rtcManager;
}

SDCardManager& SAB::getSdCardManager()
{
  return _sdCardManager;
}

ConnectivityManager& SAB::getConnectivityManager()
{
  return *_connectivityManager;
}

/*WEBServerManager& SAB::getWebServerManager()
{
  return _webServerManager;
}*/

WEBServer<WEBClient>& SAB::getWebServer()
{
  return _webServer;
}

FTPServer<FTPClient>& SAB::getFtpServer()
{
  return _ftpServer;
}

IOManager& SAB::getIoManager()
{
  return _ioManager;
}

TaskSchedulerManager& SAB::getTaskSchedulerManager()
{
  return _taskSchedulerManager;
}

PowerManager& SAB::getPowerManager()
{
  return _powerManager;
}

ScreenConfig SAB::getScreenConfig() const
{
  return _screenConfig;
}

PinMapping SAB::getPinConfig() const
{
  return _pinConfig;
}

SDCardConfig SAB::getSdCardConfig() const
{
  return _sdCardConfig;
}

const char *SAB::getSoftVersion() const
{
  return SOFT_VERSION;
}

unsigned char SAB::getError() const
{
  return _error;
}
